﻿namespace Powell.Teams.PublicApi.WebClient.Helpers
{
    using System;
    using System.Linq;
    using System.Threading.Tasks;

    using System.Net.Http;
    using System.Net.Http.Headers;
    using System.Text;

    using Newtonsoft.Json;
    using Newtonsoft.Json.Linq;

    public class ApiCallHelper
    {
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="httpClient">HttpClient used to call the protected API</param>
        public ApiCallHelper(HttpClient httpClient)
        {
            HttpClient = httpClient;
        }

        protected HttpClient HttpClient { get; private set; }

        /// <summary>
        /// Calls the API with a GET.
        /// </summary>
        /// <param name="webApiUrl">URL of the web API to call.</param>
        /// <param name="accessToken">Access token used as a bearer security token to call the web API</param>
        public async Task<JObject> WebApiGetCallAsync(string webApiUrl, string accessToken)
        {
            if (!string.IsNullOrEmpty(accessToken))
            {
                SetHeaders(accessToken);

                HttpResponseMessage response = await HttpClient.GetAsync(webApiUrl);
                if (response.IsSuccessStatusCode)
                {
                    string json = await response.Content.ReadAsStringAsync();
                    JObject result = JsonConvert.DeserializeObject(json) as JObject;
                    return result;
                }
            }

            return null;
        }

        /// <summary>
        /// Calls the protected web API with a POST.
        /// </summary>
        /// <param name="webApiUrl">URL of the web API to call (supposed to return Json)</param>
        /// <param name="accessToken">Access token used as a bearer security token to call the web API</param>
        /// <param name="postModel">Model of the object to send through post request</param>
        public async Task<JObject> WebApiPostCallAsync(string webApiUrl, string accessToken, Object postModel)
        {
            if (!string.IsNullOrEmpty(accessToken))
            {
                SetHeaders(accessToken);

                HttpContent postContent = new StringContent(JsonConvert.SerializeObject(postModel),
                    Encoding.UTF8,
                    "application/json");

                HttpResponseMessage response = await HttpClient.PostAsync(webApiUrl, postContent);
                if (response.IsSuccessStatusCode)
                {
                    string json = await response.Content.ReadAsStringAsync();
                    JObject result = JsonConvert.DeserializeObject(json) as JObject;
                    return result;
                }
            }

            return null;
        }

        /// <summary>
        /// Set headers for api call
        /// </summary>
        /// <param name="accessToken">bearer for authorization header.</param>
        private void SetHeaders(string accessToken)
        {
            var defaultRequestHeaders = HttpClient.DefaultRequestHeaders;
            if (defaultRequestHeaders.Accept == null || !defaultRequestHeaders.Accept.Any(m => m.MediaType == "application/json"))
            {
                HttpClient.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
            }

            defaultRequestHeaders.Authorization = new AuthenticationHeaderValue(
                "Bearer",
                accessToken);
        }
    }
}
